@@ -6,15 +6,18 @@ our @EXPORT = qw(run cli);
 use Test::Requires qw( Directory::Scratch Capture::Tiny File::pushd );
 
 sub cli {
-    my $dir = Directory::Scratch->new();
+    my $dir = Directory::Scratch->new(CLEANUP => !$ENV{NO_CLEANUP});
     Carton::CLI::Tested->new(dir => $dir);
 }
 
 package Carton::CLI::Tested;
 use parent qw(Carton::CLI);
 
+$Carton::CLI::UseSystem = 1;
+
 use Capture::Tiny qw(capture);
-use File::pushd;
+use File::pushd ();
+use File::Path ();
 
 sub new {
     my($class, %args) = @_;
@@ -59,5 +62,10 @@ sub system_error {
     $self->{system_error};
 }
 
+sub clean_local {
+    my $self = shift;
+    File::Path::rmtree("$self->{dir}/local", 1);
+}
+
 1;
 
